#include "new_poly_window.h"

DialogWindow::DialogWindow(MainWindow *parent)
{
    m_parent = parent;
    m_degree = 2;
    m_coefficients = NULL;
    m_layout = new QVBoxLayout(this);

    setAttribute(Qt::WA_DeleteOnClose);

    setWindowTitle(QString("Define Polynomial"));

    m_descr = new QLabel(QString("Start by defining degree of new polynomial:"));
    m_layout->addWidget(m_descr);
    m_descr->setAlignment(Qt::AlignBottom);

    m_degree_spinbox = new QSpinBox();
    m_degree_spinbox->setValue(m_degree);
    m_degree_spinbox->setMinimum(1);
    m_degree_spinbox->setMaximum(10);
    m_layout->addWidget(m_degree_spinbox);

    QPushButton *submitButton = new QPushButton(QString("Create the Polynomial"));
    m_layout->addWidget(submitButton);
    submitButton->setFocusPolicy(Qt::NoFocus);

    newCoefficients(m_degree + 1);

    m_pos_shift = 0;

    connect(m_degree_spinbox, QOverload<int>::of(&QSpinBox::valueChanged), this, &DialogWindow::changeDegree);
    connect(submitButton, &QPushButton::clicked, this, &DialogWindow::submitReceived);

}

DialogWindow::~DialogWindow ()
{
    clearCoefficients();
    delete m_degree_spinbox;
    delete m_descr;
    delete m_layout;
}


void DialogWindow::changeDegree(int new_degree)
{
    if (new_degree != m_degree) {
        if (m_coefficients != 0) {
            clearCoefficients();
        }
        newCoefficients(new_degree + 1);
    }
    showFullScreen();
}

//  Warning! Assumes Coefficients Widgets Number Zero!
void DialogWindow::newCoefficients(int number)
{
    m_coefficients = new QDoubleSpinBox[number];
//    connect(m_degree_spinbox, &QSpinBox::editingFinished, m_parent->m_im, &QInputMethod::hide);
    connect(m_degree_spinbox, &QSpinBox::editingFinished, this, &DialogWindow::forward_focus);
    for (int i = 0; i < number; i++) {
        m_coefficients[i].setValue(0.0);
        m_coefficients[i].setDecimals(8);
        m_coefficients[i].setMinimum(-1.0e10);
        m_coefficients[i].setMaximum(1.0e10);

//            connect(m_coefficients + i, &QDoubleSpinBox::editingFinished, m_parent->m_im, &QInputMethod::hide);
            connect(m_coefficients + i, &QDoubleSpinBox::editingFinished, this, &DialogWindow::forward_focus);

        m_layout->addWidget(&m_coefficients[i]);
    }
    m_coefficients[0].setValue(1.0);
    m_degree = number - 1;

    connect(m_coefficients, QOverload<double>::of(&QDoubleSpinBox::valueChanged), this, &DialogWindow::validateBox);
}

void DialogWindow::clearCoefficients()
{
    disconnect(m_coefficients, QOverload<double>::of(&QDoubleSpinBox::valueChanged), NULL, NULL);

    for (int i = 0 ; i < m_degree ; i++)
        disconnect(m_coefficients, &QDoubleSpinBox::editingFinished, NULL, NULL);

    delete[] m_coefficients;
    m_coefficients = NULL;
}

void DialogWindow::submitReceived()
{
    m_parent->coeffs_count_m = m_degree + 1;
    delete[] m_parent->coeffs_list_m;
    m_parent->coeffs_list_m = new CC[m_degree + 1];
    for (int i = 0; i <= m_degree; i++) {
        m_parent->coeffs_list_m[i] = CC(m_coefficients[i].value(), 0.0);
    }
    m_parent->unsetReverse();
    m_parent->UpdateView();
    m_parent->showFullScreen();

    MyPlot *some_plot = new MyPlot(m_parent);
    some_plot->show();

    emit(reenable_mainw());

    disconnect(this, &DialogWindow::reenable_mainw, m_parent, &MainWindow::reenable);
    disconnect(m_parent->m_im, &QInputMethod::keyboardRectangleChanged, this, &DialogWindow::changeSize);

    close();
}

void DialogWindow::validateBox(double attempt)
{
    if (PolySolver::abs_d(attempt) < MIN_A0) {
        double temp = (attempt < 0.0) ? - MIN_A0 : MIN_A0;
        m_coefficients[0].setValue(temp);
    }
}

void DialogWindow::changeSize() {
    int height = m_parent->m_app_height;
    bool kbd_visible = m_parent->m_im->isVisible();
    int kr_height = m_parent->m_im->keyboardRectangle().height();

    if (m_pos_shift < 0)
        return;
    if (m_parent->m_im->isAnimating())
        return;

    if (kbd_visible) {
        m_pos_shift = m_parent->m_im->inputItemClipRectangle().bottomLeft().y() -
                height -
                (m_parent->m_plot_size / 7);
        if (m_pos_shift < -kr_height)
            m_pos_shift = -kr_height;
        if(m_pos_shift > -1)
            m_pos_shift = -1;

        move(0, m_pos_shift);
    } /* else {
        move(0, -m_pos_shift);
        m_pos_shift = 0;
    }   */
}

void DialogWindow::forward_focus() {
    /*
    QEvent *e = new QKeyEvent(QEvent::KeyPress, Qt::Key_Tab, Qt::NoModifier);
    qApp->postEvent(this, e);   */

    int i = 0;

    QAbstractSpinBox *current = (QAbstractSpinBox *) qApp->focusWidget();
    if (! current)
        return;

    disconnect(current, &QAbstractSpinBox::editingFinished, this, &DialogWindow::forward_focus);

    focusNextChild();
    QWidget *next = qApp->focusWidget();
    if (! next->hasFocus())
        focusNextChild();

    connect(current, &QAbstractSpinBox::editingFinished, this, &DialogWindow::forward_focus);
    m_parent->m_im->setVisible(true);

    next = qApp->focusWidget();

    if (next == (QWidget *) m_degree_spinbox) {
        identCoeff(-1);
        return;
    }
    while (next != (QWidget *) m_coefficients + i && i < m_degree) {
        i++;
    }
    identCoeff(m_degree - i);
}

void DialogWindow::identCoeff(int exp) {
    if (exp < 0) {
        m_descr->setText(QString("Change degree; reset coefficients!"));
    } else if (exp == 0) {
        m_descr->setText(QString("Setting constant term:"));
    } else if (exp > m_degree) {
        m_descr->setText(QString("(Internal error.)"));
    } else {
        m_descr->setText(QString("Setting coeff of exponent ") + QString::number(exp) + QString(" :"));
    }
}
