#include "plot.h"


MyPlot::MyPlot(MainWindow *parent) :
    QMainWindow(parent)
{

    setAttribute(Qt::WA_DeleteOnClose);
    parent->m_plotShowing = true;

    center_widget = new QWidget;
    mVBox = new QVBoxLayout(center_widget);
    i_label = new QLabel(this);
    j_label = new QLabel("Initializing...", this);
    j_label->setTextFormat(Qt::PlainText);
    mVBox->addWidget(i_label, Qt::AlignCenter);
    mVBox->addWidget(j_label, Qt::AlignLeft);
    j_label->setMargin(5);

    setCentralWidget(center_widget);

    m_plot_size = parent->m_plot_size;
    pixmap = new QPixmap(m_plot_size, m_plot_size);

    setWindowTitle(QString("Plot Of Polynomial"));

    min_x = 0.0;
    max_x = 0.0;
    min_y = 0.0;
    max_y = 0.0;

    tick_interv = 1.0;
    offset_code = 0;

    m_parent = parent;

    FindInterval();

    tick_interv = std::log10(scale_x / 5.0);    //  5.0 chosen for 1-2.5-5 scaling.
                                                //  Otherwise, 2.0 would be chosen.
    tick_interv = std::floor(tick_interv);
    tick_interv = std::exp(tick_interv * std::log(10.0));


    //  Feature:
    //  Add 1-2.5-5 scaling...
    if (scale_x / tick_interv > 25.0) {
        tick_interv *= 5.0;
    } else if (scale_x / tick_interv > 10.0) {
        tick_interv *= 2.5;
    }


    j_label->setText(QString("X-Tick-Interval: ") + QString::number(tick_interv, 'g', 2));

    point_list = new QPointF[81];

    resize(420, 420);

    double temp_x = 0.0;
    double temp_y = 0.0;

    for (int i = 0; i < 81; i++) {
        temp_x = min_x + (scale_x * i / 80);
        temp_y = m_parent->solver->at(temp_x);
        point_list[i].setX(m_plot_size * (temp_x - min_x) / scale_x);
        point_list[i].setY(m_plot_size * (max_y - temp_y) / scale_y);
    }

}

MyPlot::~MyPlot()
{
    m_parent->m_plotShowing = false;
    delete i_label;
    delete j_label;
    delete mVBox;
    delete center_widget;
    delete[] point_list;
    delete pixmap;
}

//  New feature as of September 6, 2020:
//  Perform internal math with 'long double', but
//  return answers as 'double'.
void MyPlot::FindInterval()
{
    double temp = double(m_parent->sols_list->real());
    min_x = temp;
    max_x = temp;

    for (int i = 1; i < m_parent->sols_count; i++) {
        temp = double(m_parent->sols_list[i].real());
        min_x = std::min(min_x, temp);
        max_x = std::max(max_x, temp);

        //  In case there are no real roots...
        min_y = std::min(min_y, m_parent->solver->at(temp));
        max_y = std::max(max_y, m_parent->solver->at(temp));
    }

    double span_x = max_x - min_x;

    span_x *= std::pow(0.5, m_parent->coeffs_count_m - 1);
    span_x = std::max(span_x, 0.25);

    for (double s = min_x; s < max_x; s += span_x / 25) {
        min_y = std::min(min_y, m_parent->solver->at(s));
        max_y = std::max(max_y, m_parent->solver->at(s));
    }

    double span_y = max_y - min_y;
    span_y *= 0.5;
//    span_y = std::max(span_y, 0.25);

    if (max_x - min_x < 0.05) {
        span_x = 1.25;
        span_y = std::max(span_y, 1.25);
    }

    min_x -= span_x;
    max_x += span_x;
    scale_x = max_x - min_x;

    if (min_x > 0.0) {
        offset_code = 1;
    } else if (max_x < 0) {
        offset_code = -1;
    }

    min_y -= span_y;
    max_y += span_y;
    scale_y = max_y - min_y;
}

void MyPlot::resizeEvent(QResizeEvent *event)
{
    QPainter painter;
    QPen pen1(Qt::white, 4, Qt::SolidLine);
    QPen pen2(Qt::blue, 4, Qt::SolidLine);

    QPen pen3(Qt::white, 16, Qt::SolidLine);
    pen3.setCapStyle(Qt::RoundCap);
    QBrush brush(Qt::blue);

    pixmap->fill(Qt::gray);

    painter.begin(pixmap);
    painter.setPen(pen1);
    painter.setBrush(brush);
    painter.setRenderHint(QPainter::Antialiasing, true);

    double temp = -min_x * double(m_plot_size) / scale_x;
    painter.drawLine(temp, 0.0, temp, double(m_plot_size));
    temp = (scale_y + min_y) * double(m_plot_size) / scale_y;
    painter.drawLine(0.0, temp, double(m_plot_size), temp);

    painter.setPen(pen3);
    int minus_ticks = int(min_x / tick_interv);

    QPointF text_pos;
    QFont draw_font = painter.font();
    draw_font.setPointSize(32);
    painter.setFont(draw_font);

    if (offset_code > 0) {
        minus_ticks++;
        text_pos = QPointF(((minus_ticks * tick_interv) - min_x) * double(m_plot_size) / scale_x - 12, double(temp));
        painter.drawText(text_pos, QString::number(minus_ticks * tick_interv, 'g', 3));
    }

    int max_ticks = int(max_x / tick_interv);

    if (offset_code < 0) {
        max_ticks--;
        text_pos = QPointF(((max_ticks * tick_interv) - min_x) * double(m_plot_size) / scale_x - 12, double(temp));
        painter.drawText(text_pos, QString::number(max_ticks * tick_interv, 'g', 3));
    }

    while (minus_ticks <= max_ticks) {
        painter.drawPoint(((minus_ticks * tick_interv) - min_x) * double(m_plot_size) / scale_x, temp);
        minus_ticks++;
    }

    painter.setPen(pen2);
    painter.drawPolyline(point_list, 81);
    painter.end();

    i_label->setPixmap(*pixmap);
    i_label->setAlignment(Qt::AlignCenter);
    QMainWindow::resizeEvent(event);
}

//void MyPlot::closeEvent(QCloseEvent *event) {
//    QMainWindow::closeEvent(event);
//    delete this;
//}
